import requests
from pprint import pprint
import json
import random
import argparse
import os

def main():
    parser = argparse.ArgumentParser(usage="\n Phoneinfo.py -i [number] (Command to search phone number)")
    parser.add_argument("-l","--login", metavar="string", dest="loginNumber", help="Login into truecaller [7892xxxxxx]")
    parser.add_argument("-c", "--countrycode", metavar="string", dest="code", help="Country code [+91]")
    parser.add_argument("-i", "--info", metavar="string", dest="targetNumber", help="Phone number to search [98xxxxxxxx]")

    args = parser.parse_args()

    if args.loginNumber and args.code:
        phone_number = args.code + args.loginNumber
        register_phone_number(phone_number, True)

    elif args.targetNumber and args.code:
        get_phone_info(args.targetNumber, args.code, None, True)

    elif args.loginNumber or args.targetNumber or args.code:
        print("[+] Please use like this : phoneinfo.py -l 98xxxxxxxx -c +91")
    
    else:
        print("[+] please use phoneinfo -h")



# Truecaller API handlers

def register_phone_number(phoneNumber, isCLI = False):
    NUMS = '1', '2', '3', '4', '5', '6', '7', '8', '9', '0'
    LETTS = 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'
    json = {'countryCode':'','dialingCode':None,'installationDetails':{'app':{'buildVersion':5,'majorVersion':11,'minorVersion':75, 'store':'GOOGLE_PLAY'},'device':{'deviceId':''.join(random.choices(NUMS+LETTS, k=16)),'language':'en','manufacturer':'Xiaomi','mobileServices':['GMS'],'model':'M2010J19SG','osName':'Android','osVersion':'10','simSerials':[''.join(random.choices(NUMS, k=19)), ''.join(random.choices(NUMS, k=20))]},'language':'en','sims':[{'imsi':''.join(random.choices(NUMS, k=15)),'mcc':'413','mnc':'2','operator':None}]},'phoneNumber':phoneNumber,'region':'region-2','sequenceNo':2}
    headers = {'content-type':'application/json; charset=UTF-8','accept-encoding':'gzip','user-agent':'Truecaller/11.75.5 (Android;10)','clientsecret':'lvc22mp3l1sfv6ujg83rd17btt'}

    try:
        response = requests.post('https://account-asia-south1.truecaller.com/v2/sendOnboardingOtp', headers=headers, json=json)
        if response.status_code == 200:
            requestId = response.json()["requestId"]
            if not isCLI:
                if response.json()['status'] == 1 or response.json()['status'] == 9:
                    return {
                        "status": True,
                        "message": f"Sent an OTP for {phoneNumber}",
                        "requestId": requestId
                    }
                else:
                    return {
                        "status": False,
                        "message": response.json()['message'],
                        "requestId": None
                    }
            #  For CLI users
            else:
                if response.json()['status'] == 1 or response.json()['status'] == 9:
                    otp = str(input("\n[+] Please Enter OTP ==>  : "))
                    otp_validate_status = validate_OTP(phoneNumber, otp, requestId, True)
                    if otp_validate_status:
                        pass
                    else:
                        print("\n[-] Please try again!")

                else:
                    print("\n [-] Failed to sent OTP!\n [-] Reason: "+ response.json()["message"])
        else:
            if isCLI:
                print("\n[-] Something went wrong!")
            else:
                return {
                    "status": False,
                    "message": "request not statisfied, status_code => " + str(response.status_code),
                    "requestId": None
                }
    except Exception as ex:
        if not isCLI:
            return {
                "status": False,
                "message": "Failed to send OTP! Exception occured : " + str(ex),
                "requestId": None
            }
        else:
            print("\n[-] Exception occured : Exception ==> "+ str(ex))

def validate_OTP(phoneNumber, OTP, requestId, isCLI = False):
    json = {
            'countryCode':'',
            'dialingCode':None,
            'phoneNumber':phoneNumber,
            'requestId': requestId,
            'token':OTP
        }

    headers = {
        'content-type':'application/json; charset=UTF-8',
        'accept-encoding':'gzip',
        'user-agent':'Truecaller/11.75.5 (Android;10)',
        'clientsecret':'lvc22mp3l1sfv6ujg83rd17btt'
    }

    try:
        response = requests.post('https://account-asia-south1.truecaller.com/v1/verifyOnboardingOtp', headers=headers, json=json)

        if response.json()['status'] == 11:
            if isCLI:
                print("\n[-] OTP Code is Invalid")
                return False
            else:
                return {
                    "status": False,
                    "message": "OTP code is Invalid"
                }
        
        elif response.json()['status'] == 2 and response.json()['suspended']:
            if isCLI:
                print("\n[-] Oops!.. Your account got suspended. Try another number :(")
                return False
            else:
                return {
                    "status": False,
                    "message": "Opps!... Your account got suspended. Try another number :("
                }
        else:
            if isCLI:
                with open(".token.txt", "w") as file:
                    file.write(response.json()["installationId"])
                print("\n[+] Validation success!")
                return True

            else:
                return {
                    "status": True,
                    "message": "Validation success!",
                    "access_token": response.json()["installationId"]
                }
        
    except Exception as ex:
        if isCLI:
            print("\n[-] Exception occurred : ==> " + str(ex))
        else:
            return {
                "status": False,
                "message": "Failed to validate OTP! Exception : "+ str(ex)
            }

def get_phone_info(phoeNumber, countryCode, authToken, isCLI = False):
    if authToken == None and isCLI:
        if os.path.exists(os.getcwd() + ".token.txt"):
            with open(".token.txt", "r") as file:
                data = file.read()
                authToken = data
        else:
            print("[-] Please Login and try again!")
            return 0

    if authToken == None or authToken == "":
        print("[-] Please Login and try again!")
        return 0

    params = {
        'q':phoeNumber,
        'countryCode':countryCode,
        'type':'4',
        'locAddr':'',
        'placement':'SEARCHRESULTS,HISTORY,DETAILS',
        'encoding':'json'
    }

    headers = {
        'content-type':'application/json; charset=UTF-8',
        'accept-encoding':'gzip',
        'user-agent':'Truecaller/11.75.5 (Android;10)',
        'clientsecret':'lvc22mp3l1sfv6ujg83rd17btt',
        'authorization':'Bearer ' + authToken
    }

    try:
        response = requests.get("https://search5-noneu.truecaller.com/v2/search", headers=headers, params=params)

        if not isCLI:
            if response.json().get('status'):
                return {
                    "status": False,
                    "message": "Failed to get user info"
                }
            else:
                return {
                    "status": True,
                    "message": "User details found",
                    "data": json.loads(response.content)
                }

        else:
            if response.json().get('status'):
                print("\n[+] Failed to get user data")
            else:
                with open(".data.txt","w") as file:
                    json.dump(response.json(), file, indent=4)
                with open(".data.txt", "r") as file:
                    data = json.load(file)
                pprint(data)

    except Exception as ex:
        if isCLI:
            print("[-] Exception occurred : ==> "+ str(ex))
        else:
            return {
                "status": False,
                "message": "Failed to get user details, Exception : "+ str(ex)
            }
        

if __name__ == "__main__":
    main()


from setuptools import setup, find_packages
import codecs
import os


VERSION = '1.0.0.15'
DESCRIPTION = "phoneinfopy is a Python package that allow you to get respones from truecaller APIs. You can able to do login, OTP verification, and phone number search using Truecaller API with this package."

long_descirption_content = """# PhoneInfopy

PhoneInfopy is a python tool to directly make request to Truecaller API's and get response. This module will allow you to register your number, validate OTP while registering and search information about a phone number.

**Table to Contents**
- [PhoneInfoPy](#PhoneInfoPy)
    - [Requirements](#requirements)
    - [Command Line Usage](#Command-line)
        - [Installation](#Installation)
        - [Login](#Login)
        - [Search Number](#search-number)
    - [Module usage](#Module-usage)
        - [Installation](#Install)
        - [Register](#Register)
        - [Validate OTP](#Validate-OTP)
        - [Search phone number](#Search-number)

## Requirements
To use PhoneInfopy module you need below pre requirements.
- valid mobile number
- python
- truecaller auth token [This token will get by login/register process]

## Command Line Usage
You can install PhoneInfopy package using pip

- `phoneinfopy -h [--help]` for help
- `phoneinfopy -l [--login]` for login [Need to mention country code]
- `phoneinfopy -c [--code]` for country code
- `phoneinfopy -i [--info]` for search mobile number [Need to mention country code]

### Installation
```bash
pip install phoneinfopy
```

### Login
```bash
python3 -m phoneinfopy.cli -c <countryCode> -l <phoneNumber>
```
#### Example
```bash
python3 -m phoneinfopy.cli -c +91 -l 98xxxxxxxx
```

## search-Number
You can search phone number by using below command
```bash
python3 -m phoneinfo.cli -c +91 -i 98xxxxxxxx
```

## Module-usage

### Install
You can install the phoneinfopy moudle using pip:
```bash
pip install phoneinfopy
```

## Register
```python
from phoneinfopy import register_phone_number

number = "+9198xxxxxxxx"
response = register_phone_number(number)

# register_phone_number method will return json object
# example:
# {
#   status : bool,
#   message: str
#   requestId: str
# }
```

## Validate-OTP:
Once you get success response from register method next you need to verify the OTP to get access token.


```python
from phoneinfopy import validate_OTP

number = "+9198xxxxxxxx"
OTP = "123456"
requestId = "<you get this ID while register process>"
response = validate_OTP(number, OTP, requestId)

# validate_OTP method will return json object
# {
#   status: bool
#   message: str
#   access_token: str  [If OTP validatino is success]
# }
```

## Search-number
You need to get user information using phone number you can use like this..
```python
from phoneinfopy import get_phone_info

target_number = "98xxxxxxxx"
country_code = "+91"
access_token = "a10--******" ["you get this token while validating OTP"]

response = get_phone_info(target_number, country_code, access_token)

# get_phone_info will return json object 
#{
#   status: bool
#   message: str
#   data: object [if user info found]
# }
```

## Contribution

Contributions to the phoneinfopy package are welcome!...
If you faced any issue or you have good idea to improve please open a issue or submit a pull request.

## Let's Connect:
- [Instagram](https://www.instagram.com/mr_3rr0r_501/)
- [LinkedIn](https://www.linkedin.com/in/yuvaraj-a-57a103171)
- [Youtube](https://youtube.com/@Mr3rr0r501)

"""

# Setting up
setup(
    name="phoneinfopy",
    version=VERSION,
    author="Mr_3rr0r_501 [yuvaraj]",
    author_email="<yuvarajucet@gmail.com>",
    description=DESCRIPTION,
    long_description= long_descirption_content,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=['requests'],
    keywords=['truecaller','phoneinfo', 'numberinfo', 'phonenumberinfo'],
	classifiers = [
	    "Topic :: Utilities",
	    "Natural Language :: English",
	    "Operating System :: OS Independent",
	    "Programming Language :: Python :: 3",
	]
)
